@@ -1,21 +1,18 @@
 package HTTP::Engine::Interface::CGI;
-use strict;
-use warnings;
-use base 'HTTP::Engine::Plugin';
-use HTTP::Engine::Role;
+use Moose;
 with 'HTTP::Engine::Role::Interface';
-
 use constant should_write_response_line => 0;
 
-sub run :Method{
-    my ($self, $c) = @_;
-    local %ENV = %ENV;
-    $c->handle_request;
+sub run {
+    my ($self) = @_;
+    $self->handle_request();
 }
 
 1;
 __END__
 
+=for stopwords 
+
 =for stopwords CGI Naoki Nyarla Okamura yaml
 
 =head1 NAME
@@ -24,14 +21,24 @@ HTTP::Engine::Interface::CGI - CGI interface for HTTP::Engine
 
 =head1 SYNOPSIS
 
-  interface:
-    module: CGI
-    request_handler: methodname
+    HTTP::Engine::Interface::CGI->new();
+
+=head1 METHODS
+
+=over 4
+
+=item run
+
+internal use only
+
+=back
 
 =head1 AUTHOR
 
 Naoki Okamura (Nyarla) E<lt>thotep@nyarla.netE<gt>
 
+Tokuhiro Matsuno
+
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify
