@@ -1,29 +1,68 @@
 package HTTP::Engine::Interface::FCGI;
-use strict;
-use warnings;
-use base 'HTTP::Engine::Plugin';
-use HTTP::Engine::Role;
+use Moose;
 with 'HTTP::Engine::Role::Interface';
-
-use HTTP::Status;
-use FCGI;
-
 use constant should_write_response_line => 0;
+use FCGI;
+use UNIVERSAL::require;
+
+has leave_umask => (
+    is      => 'ro',
+    isa     => 'Bool',
+    default => 0,
+);
+
+has keep_stderr => (
+    is      => 'ro',
+    isa     => 'Bool',
+    default => 0,
+);
+
+has nointr => (
+    is      => 'ro',
+    isa     => 'Bool',
+    default => 0,
+);
+
+has detach => (
+    is      => 'ro',
+    isa     => 'Bool',
+    default => 0,
+);
+
+has manager => (
+    is      => 'ro',
+    isa     => 'Str',
+    default => "FCGI::ProcManager",
+);
+
+has nproc => (
+    is      => 'ro',
+    isa     => 'Int',
+    default => 1,
+);
+
+has pidfile => (
+    is      => 'ro',
+    isa     => 'Str',
+);
+
+has listen => (
+    is  => 'ro',
+    isa => 'Str',
+);
 
 sub run {
-    my ( $self, $class, $listen, ) = @_;
-
-    my $options = $self->config;
+    my ( $self, ) = @_;
 
     my $sock = 0;
-    if ($listen) {
+    if ($self->listen) {
         my $old_umask = umask;
-        unless ( $options->{leave_umask} ) {
+        unless ( $self->leave_umask ) {
             umask(0);
         }
-        $sock = FCGI::OpenSocket( $listen, 100 )
+        $sock = FCGI::OpenSocket( $self->listen, 100 )
           or die "failed to open FastCGI socket; $!";
-        unless ( $options->{leave_umask} ) {
+        unless ( $self->leave_umask ) {
             umask($old_umask);
         }
     }
@@ -32,42 +71,36 @@ sub run {
           or die "STDIN is not a socket; specify a listen location";
     }
 
-    $options ||= {};
-
     my %env;
     my $error = \*STDERR;    # send STDERR to the web server
     $error = \*STDOUT                # send STDERR to stdout (a logfile)
-      if $options->{keep_stderr};    # (if asked to)
+      if $self->keep_stderr;         # (if asked to)
 
     my $request =
       FCGI::Request( \*STDIN, \*STDOUT, $error, \%env, $sock,
-        ( $options->{nointr} ? 0 : &FCGI::FAIL_ACCEPT_ON_INTR ),
+        ( $self->nointr ? 0 : &FCGI::FAIL_ACCEPT_ON_INTR ),
       );
 
     my $proc_manager;
 
-    if ($listen) {
-        $options->{manager} ||= "FCGI::ProcManager";
-        $options->{nproc}   ||= 1;
+    if ($self->listen) {
+        $self->daemon_fork() if $self->detach;
 
-        $self->daemon_fork() if $options->{detach};
-
-        if ( $options->{manager} ) {
-            eval "use $options->{manager}; 1" or die $@; ## no critic
-
-            $proc_manager = $options->{manager}->new(
+        if ( $self->manager ) {
+            $self->manager->use or die $@;
+            $proc_manager = $self->manager->new(
                 {
-                    n_processes => $options->{nproc},
-                    pid_fname   => $options->{pidfile},
+                    n_processes => $self->nproc,
+                    pid_fname   => $self->pidfile,
                 }
             );
 
             # detach *before* the ProcManager inits
-            $self->daemon_detach() if $options->{detach};
+            $self->daemon_detach() if $self->detach;
 
             $proc_manager->pm_manage();
         }
-        elsif ( $options->{detach} ) {
+        elsif ( $self->detach ) {
             $self->daemon_detach();
         }
     }
@@ -83,13 +116,29 @@ sub run {
         }
 
         local %ENV = %env;
-        $class->handle_request();
+        $self->handle_request();
 
         $proc_manager && $proc_manager->pm_post_dispatch();
     }
 }
 
-sub write {
+sub daemon_fork {
+    require POSIX;
+    fork && exit;
+}
+
+sub daemon_detach {
+    my $self = shift;
+    print "FastCGI daemon started (pid $$)\n";
+    open STDIN,  "+</dev/null" or die $!; ## no critic
+    open STDOUT, ">&STDIN"     or die $!;
+    open STDERR, ">&STDIN"     or die $!;
+    POSIX::setsid();
+}
+
+
+use HTTP::Engine::ResponseWriter;
+HTTP::Engine::ResponseWriter->meta->add_method( _write => sub {
     my($self, $buffer) = @_;
 
     unless ( $self->{_prepared_write} ) {
@@ -104,33 +153,54 @@ sub write {
     # FastCGI does not stream data properly if using 'print $handle',
     # but a syswrite appears to work properly.
     *STDOUT->syswrite($buffer);
-}
-
-sub daemon_fork {
-    require POSIX;
-    fork && exit;
-}
-
-sub daemon_detach {
-    my $self = shift;
-    print "FastCGI daemon started (pid $$)\n";
-    open STDIN,  "+</dev/null" or die $!; ## no critic
-    open STDOUT, ">&STDIN"     or die $!; ## no critic
-    open STDERR, ">&STDIN"     or die $!; ## no critic
-    POSIX::setsid();
-}
+});
 
 1;
 __END__
 
+=for stopwords nointr pidfile nproc
+
+=head1 NAME
+
+HTTP::Engine::Interface::FCGI - FastCGI interface for HTTP::Engine
 
 =head1 SYNOPSIS
 
-  interface:
-    module: FCGI
-      args:
-        leave_umask: 1
-      request_handler: methodname
+    HTTP::Engine::Interface::FCGI->new(
+    );
+
+=head1 ATTRIBUTES
+
+=over 4
+
+=item leave_umask
+
+=item keep_stderr
+
+=item nointr
+
+=item detach
+
+=item manager
+
+=item nproc
+
+=item pidfile
+
+=item listen
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item run
+
+internal use only
+
+=back
+
 
 =head1 AUTHORS
 
@@ -138,5 +208,5 @@ Tokuhiro Matsuno
 
 =head1 THANKS TO
 
-may codes copied from L<Catalyst::Engine::FastCGI>. thanks authors of C::E::FastCGI!
+many codes copied from L<Catalyst::Engine::FastCGI>. thanks authors of C::E::FastCGI!
 
